;DGE (DynamicGameEngine) by GEM

#include <GDIPlus.au3>
#include <Misc.au3>

_GDIPlus_Startup()
Global $user32dll = DllOpen("user32.dll")
OnAutoItExitRegister("__DGE_Close")

Func __DGE_Close()
	DllClose($user32dll)
	_GDIPlus_Shutdown()
EndFunc   ;==>__DGE_Close

Func _DGE_CreateWindow($title, $width, $height, $exitfunction = "close")
	;create
	Opt("GUIOnEventMode", 1)
	Local $gui = GUICreate($title, $width, $height)
	GUISetOnEvent(-3, $exitfunction, $gui)
	Local $graphics = _GDIPlus_GraphicsCreateFromHWND($gui)
	Local $bitmap = _GDIPlus_BitmapCreateFromGraphics($width, $height, $graphics)
	Local $buffer = _GDIPlus_ImageGetGraphicsContext($bitmap)
	GUISetState(@SW_SHOW, $gui)
	;end
	Local $return[4] = [$gui, $graphics, $bitmap, $buffer]
	Return $return
EndFunc   ;==>_DGE_CreateWindow

Func _DGE_UpdateWindow(ByRef $handle)
	;check
	If Not IsArray($handle) Then Return -1
	;size
	If UBound($handle) < 4 Then Return -2
	;dispose
	_GDIPlus_GraphicsDrawImage($handle[1], $handle[2], 0, 0)
	;end
	Return 0
EndFunc   ;==>_DGE_UpdateWindow

Func _DGE_ClearWindow(ByRef $handle)
	;check
	If Not IsArray($handle) Then Return -1
	;size
	If UBound($handle) < 4 Then Return -2
	;dispose
	_GDIPlus_GraphicsClear($handle[3])
	;end
	Return 0
EndFunc   ;==>_DGE_ClearWindow


Func _DGE_DisposeWindow(ByRef $handle)
	;check
	If Not IsArray($handle) Then Return -1
	;size
	If UBound($handle) < 4 Then Return -2
	;dispose
	_GDIPlus_GraphicsDispose($handle[3])
	_GDIPlus_BitmapDispose($handle[2])
	_GDIPlus_GraphicsDispose($handle[1])
	GUIDelete($handle[0])
	;end
	Return 0
EndFunc   ;==>_DGE_DisposeWindow

Func _DGE_MoveDAP(ByRef $player, ByRef $map, ByRef $collision)
	;check
	If Not IsArray($player) Then Return -1
	;move
	For $i = 0 To UBound($player) - 1
		Switch $player[$i][4]
			Case 0
				Select
					Case _IsPressed($player[$i][5], $user32dll) And __DGE_Collision($player[$i][0] + 1, $player[$i][1], $map, $collision) = False
						$player[$i][4] = 1
					Case _IsPressed($player[$i][6], $user32dll) And __DGE_Collision($player[$i][0] - 1, $player[$i][1], $map, $collision) = False
						$player[$i][4] = 2
					Case _IsPressed($player[$i][7], $user32dll) And __DGE_Collision($player[$i][0], $player[$i][1] + 1, $map, $collision) = False
						$player[$i][4] = 3
					Case _IsPressed($player[$i][8], $user32dll) And __DGE_Collision($player[$i][0], $player[$i][1] - 1, $map, $collision) = False
						$player[$i][4] = 4
				EndSelect
			Case 1
				If $player[$i][2] >= $player[$i][9] Then
					$player[$i][0] += 1
					$player[$i][2] = 0
					$player[$i][4] = 0
					If _IsPressed($player[$i][5], $user32dll) And __DGE_Collision($player[$i][0] + 1, $player[$i][1], $map, $collision) = False Then $player[$i][4] = 1
				Else
					$player[$i][2] += $player[$i][10]
				EndIf
			Case 2
				If $player[$i][2] <= 0 - $player[$i][9] Then
					$player[$i][0] -= 1
					$player[$i][2] = 0
					$player[$i][4] = 0
					If _IsPressed($player[$i][6], $user32dll) And __DGE_Collision($player[$i][0] - 1, $player[$i][1], $map, $collision) = False Then $player[$i][4] = 2
				Else
					$player[$i][2] -= $player[$i][10]
				EndIf
			Case 3
				If $player[$i][3] >= $player[$i][9] Then
					$player[$i][1] += 1
					$player[$i][3] = 0
					$player[$i][4] = 0
					If _IsPressed($player[$i][7], $user32dll) And __DGE_Collision($player[$i][0], $player[$i][1] + 1, $map, $collision) = False Then $player[$i][4] = 3
				Else
					$player[$i][3] += $player[$i][10]
				EndIf
			Case 4
				If $player[$i][3] <= 0 - $player[$i][9] Then
					$player[$i][1] -= 1
					$player[$i][3] = 0
					$player[$i][4] = 0
					If _IsPressed($player[$i][8], $user32dll) And __DGE_Collision($player[$i][0], $player[$i][1] - 1, $map, $collision) = False Then $player[$i][4] = 4
				Else
					$player[$i][3] -= $player[$i][10]
				EndIf
		EndSwitch
	Next
	;end
	Return 0
EndFunc   ;==>_DGE_MoveDAP

Func __DGE_Collision($x, $y, ByRef $map, ByRef $collision)
	;check
	If Not IsArray($map) Or Not IsArray($collision) Then Return False
	;collision
	If $x < 0 Or $y < 0 Or $x = UBound($map) Or $y = UBound($map, 2) Then Return True
	For $i = 0 To UBound($collision) - 1
		If $map[$x][$y][0] = $collision[$i] Then Return True
	Next
	;end
	Return False
EndFunc   ;==>__DGE_Collision

Func _DGE_DrawDAP($graphics, ByRef $map, ByRef $info, ByRef $player)
	;check
	If Not IsArray($map) Or Not IsArray($info) Or Not IsArray($player) Then Return -1
	;local
	If UBound($info) < 7 Then Return -2
	Local $x = $info[0], $y = $info[1], $w = $info[2], $h = $info[3], $d = $info[4], $xd = $info[5], $yd = $info[6]
	;error
	If $x + $w > UBound($map, 2) Or $x < 0 Or $y + $h > UBound($map) Or $y < 0 Then Return -2
	;drawplayer
	Local $xd1, $yd1
	For $i = 0 To UBound($player) - 1
		If $player[$i][0] >= $x And $player[$i][0] < $x + $w And $player[$i][1] >= $y And $player[$i][1] < $y + $h Then
			$xd1 = 0
			$yd1 = 0
			If $xd = 0 And $yd = 0 Then
				$xd1 = $player[$i][2]
				$yd1 = $player[$i][3]
			EndIf
			_GDIPlus_GraphicsFillEllipse($graphics, ($player[$i][0] - $x) * $d + $xd1, ($player[$i][1] - $y) * $d + $yd1, $d, $d)
		EndIf
	Next
	;end
	Return 0
EndFunc   ;==>_DGE_DrawDAP

Func _DGE_DrawDAM($graphics, ByRef $map, ByRef $texturen, ByRef $info)
	;check
	If Not IsArray($map) Or Not IsArray($texturen) Or Not IsArray($info) Then Return -1
	;local
	If UBound($info) < 7 Then Return -2
	Local $x = $info[0], $y = $info[1], $w = $info[2], $h = $info[3], $d = $info[4], $xd = $info[5], $yd = $info[6]
	;error
	If $x + $w > UBound($map, 2) Or $x < 0 Or $y + $h > UBound($map) Or $y < 0 Then Return -2
	;draw
	Local $x1 = 0, $y1 = 0
	For $a = $x To $x + $w - 1
		For $b = $y To $y + $h - 1
			_GDIPlus_GraphicsDrawImageRect($graphics, $map[$a][$b][1], $x1 - $xd, $y1 - $yd, $d, $d)
			$y1 += $d
		Next
		$x1 += $d
		$y1 = 0
	Next
	;end
	Return 0
EndFunc   ;==>_DGE_DrawDAM

Func _DGE_LoadDAM($filename, ByRef $map, ByRef $texturen, ByRef $collision)
	;check
	If Not IsArray($map) Or Not IsArray($texturen) Or Not IsArray($collision) Then Return -1
	;file
	If Not FileExists($filename) Then Return -2
	Local $file = FileOpen($filename)
	Local $read = FileRead($file)
	FileClose($file)
	;split
	Local $split = StringSplit($read, "|", 2)
	If UBound($split) < 3 Then Return -3
	;texturen
	Local $split2 = StringSplit($split[0], ";", 2)
	If Not IsArray($split2) Then Return -4
	ReDim $texturen[$split2[0]][3]
	For $i = 0 To UBound($texturen) - 1
		Local $split3 = StringSplit($split2[$i + 1], "=", 2)
		If UBound($split3) < 2 Then ContinueLoop
		$texturen[$i][0] = $split3[0]
		If FileExists(@ScriptDir & "\" & $split3[1]) Then
			$texturen[$i][1] = _GDIPlus_ImageLoadFromFile(@ScriptDir & "\" & $split3[1])
		Else
			$texturen[$i][1] = 0
		EndIf
		$texturen[$i][2] = $split3[1]
	Next
	;map
	Local $split2 = StringSplit($split[1], ";", 2)
	If Not IsArray($split2) Then
		_DGE_UnloadDAM($map, $texturen, $collision)
		Return -6
	EndIf
	If StringLen($split2[2]) < $split2[0] * $split2[1] Then
		_DGE_UnloadDAM($map, $texturen, $collision)
		Return -7
	EndIf
	ReDim $map[$split2[0]][$split2[1]][2]
	Local $pic, $pos = 1
	For $a = 0 To UBound($map) - 1
		For $b = 0 To UBound($map, 2) - 1
			$map[$a][$b][0] = StringMid($split2[2], $pos, 1)
			$pic = 0
			For $i = 0 To UBound($texturen) - 1
				If $texturen[$i][0] = $map[$a][$b][0] Then
					$pic = $texturen[$i][1]
					ExitLoop
				EndIf
			Next
			$map[$a][$b][1] = $pic
			$pos += 1
		Next
	Next
	;collision
	Local $split2 = StringSplit($split[2], ";", 2)
	If $split2[0] = "" Then Return 1
	ReDim $collision[$split2[0]]
	For $i = 1 To UBound($split2) - 1
		$collision[$i - 1] = $split2[$i]
	Next
	;end
	Return 0
EndFunc   ;==>_DGE_LoadDAM


Func _DGE_UnloadDAM(ByRef $map, ByRef $texturen, ByRef $collision)
	;check
	If Not IsArray($map) Or Not IsArray($texturen) Then Return -1
	;texturen
	For $i = 0 To UBound($texturen) - 1
		If $texturen[$i][1] <> 0 Then _GDIPlus_ImageDispose($texturen[$i][1])
	Next
	Dim $texturen[1][3]
	;map
	Dim $map[1][1][1]
	;collision
	Dim $collision[1]
	;end
	Return 0
EndFunc   ;==>_DGE_UnloadDAM
